#include <stdio.h>
#include "ScratchRecord.h"

typedef short word;
typedef unsigned char byte;

#pragma pack(push, 1)
struct t_stat
{
  word line;                /* Current line       2*/
  word lines;               /* Total lines        2*/
  word buf_size;            /* Buffer size        2*/
  byte pen_type;            /* Draw/erase/correct 1*/
  byte pen_mesh;            /* Solid/mesh         1*/
  byte pen_size;            /* Width of pen       1*/
  byte eraser_size;         /* Width of eraser    1*/
  /* end of ver1 header                   Total: 10 bytes*/
  byte ver2sign[5];         /* Sign of IS>=ver2   5*/
  byte ver2ver;             /* Version number     1*/
  byte draw_style;          /* Draw style         1*/
  byte erase_style;         /* Erase style        1*/
  /* end of ver2 header                   Total:  8 bytes*/     
  byte extension[120];
}; /* Total 138 bytes*/

struct BMPHead
{
	char sign[2];
	int  fsize;
	int  blank;
	int  pic_start;
	int  info_size;
	int  width;
	int  height;
	WORD plane;
	WORD bpp;
	int  compress;
	int  data_size;
	int  x_res;
	int  y_res;
	int  use_col;
	int  imp_col;
};
#pragma pack(pop)

int decompressLine(unsigned char* dest, unsigned char* src)
{
  unsigned char c=src[0];
  int i,j,k,l=0;
  if ((c&0xE0)==0xE0)
  {
    unsigned char len=c&0x1F;
    unsigned char col=1;//black
    unsigned char left=160;
    unsigned char bleft=8;
    unsigned char cbyte=0;
    unsigned char locLen;

    if (len==1)
    {
      for(i=0; i<20; i++)
        dest[i]=0xFF; // black
      return 1;
    }

    j=1;
    locLen=src[j];
    do{
      for(k=0; k<locLen; k++)
      {
        cbyte<<=1;
        cbyte|=col;
        bleft--;
        if (bleft==0)
        {
          dest[l]=cbyte;
          l++;
          cbyte=0;
          bleft=8;
        }
      }
      col^=1;
      j++;
      left-=locLen;
      if (j<len)
      {
        locLen=src[j];
      }
      else
      {
        locLen=left;
      }
    } while(left);
    return len;
  }
  else
  {
    for(i=0; i<20; i++)
     dest[i]=~src[i+1]; //black
    return 21;
  }
}

void decompress(unsigned char* dest, unsigned char* src, int lines)
{
	for(int i=0; i<lines; i++)
	{
		if (*src==0xFF)
		{
			for(int j=0; j<(lines-i)*20; j++)
				*(dest++)=0xFF;
			return;
		}
		src+=decompressLine(dest,src);
		dest+=20;
	}
}

ScratchRecord::ScratchRecord()
:lines(0),bits(NULL),wBMP(NULL)
{
}

ScratchRecord::ScratchRecord(AdtRecord &rec)
{
	unsigned char* point=rec.getData();
	t_stat* head=(t_stat*)point;
	lines=head->lines;
	bits=new unsigned char[lines*20];
	decompress(bits,point+sizeof(t_stat),lines);
	wBMP=CreateBitmap(160,20,1,1,bits);
}

ScratchRecord::~ScratchRecord()
{
	if (wBMP!=NULL)
		DeleteObject(wBMP);
	if (bits!=NULL)
		delete [] bits;	  
}

void ScratchRecord::setRecord(AdtRecord& rec)
{
	if (bits)
	{
		delete [] bits;
		bits=NULL;
		wBMP=NULL;
	}
	unsigned char* point=rec.getData();
	t_stat* head=(t_stat*)point;
	lines=head->lines;
	bits=new unsigned char[lines*20];
	decompress(bits,point+sizeof(t_stat),lines);
	wBMP=CreateBitmap(160,20,1,1,bits);
}

int ScratchRecord::getHeight()
{
        return lines;
}

int ScratchRecord::saveBMP(const char *fname)
{
	BMPHead bmp_head={{'B','M'},0,0,62,40,160,0,1,1,0,0,0,0,0,0};
	unsigned char bmp_pal[2][4]=
	{/*  B   G   R   I*/
	  {  0,  0,  0,  0}, //0
	  {255,255,255,  0}  //1
	};
	FILE* file;
	int i;

	file=fopen(fname,"wb");
	if (!file)
	{
 		return 0;
	}
	bmp_head.height=lines;
	bmp_head.data_size=lines*20;
	bmp_head.fsize=62+bmp_head.data_size;

	fwrite(&bmp_head,1,sizeof(bmp_head),file);
	fwrite(&bmp_pal,1,sizeof(bmp_pal),file);
	for(i=lines-1; i>=0; i--)
		fwrite(bits+i*20,1,20,file);
	fclose(file);
	return 1;
}

HBITMAP ScratchRecord::getFullBMP()
{
	return CreateBitmap(160,lines,1,1,bits);
}

HBITMAP ScratchRecord::getBitmap()
{
	return wBMP;
}

ChainBitmap* ScratchRecord::createBitmap()
{
	return new ChainBitmap(160,lines,bits);
}